# -*- encoding : utf-8 -*-
require 'fakecrm/resource/extensions/kinds'

require 'fakecrm/resource/account'
require 'fakecrm/resource/contact'
require 'fakecrm/resource/custom_type'

module Fakecrm
  class Activity
    include DataMapper::Resource

    include Kinds

    belongs_to :account, :required => false
    belongs_to :contact, :required => false

    property :id,                     Serial
    property :title,                  String, :required => true
    property :appointment_dtend_at,   DateTime
    property :appointment_dtstart_at, DateTime
    property :appointment_location,   String
    property :email_cc,               String
    property :state,                  String
    property :priority,               Integer
    property :tags,                   CommaSeparatedList
    property :created_at,             DateTime
    property :changed_at,             DateTime
    property :updated_at,             DateTime
    property :deleted_at,             ParanoidDateTime

    property :comments,               Json 

    property :updated_by,             String, :default => 'root'


    validates_with_method :state, :method => :check_state
    validates_presence_of :title

    def custom_type
      CustomType.get!(self.kind) unless self.kind.nil?
    end

    def check_state
      custom_type = self.custom_type
      if !custom_type.nil? && !custom_type.states.nil? && !custom_type.states.empty?
        if self.state.nil? || !custom_type.states.include?(self.state)
          # FIXME: localization
          return [false, 'ist kein gültiger Wert']
        end
      end

      return true
    end

    before :save do
      self.comments = [] if self.comments.nil?
    end

    def self.transform_comment_params( params )
      if params["comment_notes"]
        if params["comment_contact_id"]
          contact = Contact.get!( params["comment_contact_id"].to_i )
        else
          contact = Contact.search(:params => {:login => 'root'}).first
        end
        new_comment = {
          :contact_id => contact.id,
          :updated_by => 'root',
          :notes => params["comment_notes"].to_s,
          :published => !!params["comment_published"],
          :updated_at => Time.now
        }
        params["comments"] = (Activity.get!(params["id"].to_i).comments || []) rescue []
        params["comments"] << new_comment
      end
      params.delete("comment_notes")
      params.delete("comment_contact_id")
      params.delete("comment_published")

      return params
    rescue ::DataMapper::ObjectNotFoundError
      status 404
    end

  end
end

