# -*- encoding : utf-8 -*-
require 'fakecrm/resource/extensions/type_extender'

# workaround circular reference
module Fakecrm
  class CustomType
    include DataMapper::Resource

    property :id,               String, :unique => true, :key => true # default will not work here
    property :name,             String, :required => true, :length => 20
  end
end

require 'fakecrm/resource/custom_attribute'

# workaround circular reference
module Fakecrm
  class CustomType

    property :kind,             Enum["Activity", "Account", "Event", "Contact", "EventContact"], :required => true
    property :icon_css_class,   String

    # only one entry for 'Contact' possible
    validates_uniqueness_of     :kind, :if => lambda { |c| c.kind == 'Contact' }

    # only for activity && event_contact
    property :states,            CommaSeparatedList

    has n, :custom_attributes

    def activity?
      self.kind == 'Activity'
    end

    # setting name for the first time also sets id
    def name=(new_name)
      if self.id.nil?
        if self.kind == "EventContact"
          self.id = SecureRandom.hex
        else
          self.id = new_name
        end
      end

      super
    end

    # This is neccessary because updating DM associations is BROKEN
    def custom_attributes=(custom_attributes)
      if custom_attributes
        existing_attributes = self.custom_attributes.all
        existing_attributes.each do |existing_attribute|
          if !(custom_attributes.find {|c| (c[:name] || c["name"]) == existing_attribute.name})
            existing_attribute.destroy
          end
        end

        custom_attributes.each do |custom_attribute|
          name = custom_attribute["name"] || custom_attribute[:name]
          c = self.custom_attributes.first(:name => name)
          if c
            c.attributes = custom_attribute
          else
            self.custom_attributes.new(custom_attribute)
          end
        end
        @dm_is_broken = true
      end
      attribute_set(:custom_attributes, self.custom_attributes)
    end

    after :save, :propagate_type_extension

    def propagate_type_extension
      ::Fakecrm.logger.debug "Installing custom attributes@ kind=#{self.kind} name=#{self.name}"
      ::Fakecrm::TypeExtender.new(self).extend!
    end

    def dirty_self?
      # This forces DM to save after changing custom attributes
      # We need this for our callback
      # Did I mention that DM associations ARE BROKEN?
      @dm_is_broken || super
    ensure
      @dm_is_broken = false # NOT REALLY
    end

    class << self
      def virtual_properties
        [:custom_attributes]
      end
    end
  end
end


