# -*- encoding : utf-8 -*-
require 'fakecrm/resource/extensions/kinds'

require 'fakecrm/resource/custom_type'

module Fakecrm
  class Event
    include DataMapper::Resource

    include Kinds

    property :id,               Serial
    property :title,            String,   :required => true, :message => 'muss ausgefüllt werden'

    property :dtstart_at,       DateTime, :required => true
    property :dtend_at,         DateTime, :required => true

    property :event_set,        String
    property :location,         String

    property :created_at,       DateTime
    property :changed_at,       DateTime
    property :updated_at,       DateTime
    property :deleted_at,       ParanoidDateTime

    property :updated_by,       String, :default => 'root'

    class << self
      def virtual_properties
        [:custom_attributes]
      end
    end

    def custom_type
      CustomType.first_or_create(:kind => 'EventContact', :name => self.kind)
    end

    def custom_attributes
      custom_type ? custom_type.custom_attributes : []
    end

    def custom_attributes=(new_attributes)
      custom_type.update({"custom_attributes"=>new_attributes})
    end

    def attributes=(attributes)
      custom_attributes = attributes.delete("custom_attributes") || attributes.delete(:custom_attributes) || []
      super(attributes) && (custom_attributes.empty? || (self.custom_attributes = custom_attributes))
    end
  end
end

