# -*- encoding : utf-8 -*-
require 'fakecrm/resource/custom_type'

module Fakecrm
  class ResourceView 
    class << self
      def decorate(source, options={})
        if source.class.include?(::Enumerable)
          source.map {|item| self.new(item, options)}
        else
          self.new(source, options)
        end
      end
    end

    def initialize(resource, options={})
      self.resource = resource

      self.custom_attributes = get_custom_attributes(self.resource)
      self.properties = get_resource_properties(self.resource)

      self.include = options.fetch(:include, :all)
      self.exclude = options.fetch(:exclude, [])
      self.additional = options.fetch(:methods, get_additional_methods(self.resource))
    end


    def to_json(*args)
      return as_json.to_json
    end

    def as_json(*args)
      result = {}

      self.properties.each do |property|
        if available?(property) && include?(property)
          result[property] = resource.send(property)
        end
      end

      self.additional.each do |method|
        result[method] = resource.send(method)
      end

      return result
    end

    def available?(property)
      if property =~ /^custom/
        self.custom_attributes.include?(property)
      else
        true
      end
    end

    def include?(property)
      !self.exclude.include?(property) && (self.include == :all || self.include.include?(property))
    end

    def exclude?(property)
      !include?(property)
    end

    def get_additional_methods(resource)
      if resource.class.respond_to? :virtual_properties
        resource.class.virtual_properties || []
      else
        []
      end
    end

    def get_resource_properties(resource)
      resource.class.properties.map(&:name)
    end

    def get_custom_attributes(resource)
      if resource.respond_to?(:one_of_a_kind?) && resource.one_of_a_kind?
        CustomType.get!(resource.kind).custom_attributes.map { |attr| :"custom_#{attr.name}" }
      else
        get_resource_properties(resource).select {|property| property =~ /^custom/ }
      end
    end

    protected
    attr_accessor :properties, :resource, :custom_attributes
    attr_accessor :include, :exclude, :additional

  end
end

